<?php

use Chekonline\Cashbox\Commands\BatchCommand;
use Chekonline\Cashbox\Commands\NoOperationCommand;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;


class shopChekonlinePluginBackendTestController extends waJsonController
{

    public function execute()
    {
        $error = 0;
        $result = 'Соединение установлено';
        /** @var shopChekonlinePlugin $plugin */
        $plugin = wa()->getPlugin('chekonline');
        try {

            $plugin->init();
            $requestId = strval(time());

            $clientId = $plugin->getSettings('chekonline_client_id');

            $batchCommand = new BatchCommand($requestId);
            $batchCommand->setClientId($clientId)
                         ->setQueueLen(100)
                         ->setDevice('auto');
            $noOperationCommand = new NoOperationCommand($requestId);
            $noOperationCommand->setPassword(intval($plugin->getSettings('chekonline_cashier_password')))
                               ->setClientId($clientId);

            $noOperationCommand->validate();
            $batchCommand->addCommandForExecute($noOperationCommand);

            $batchCommand->validate();

            $plugin->client->validate();

            $response = $plugin->client->executeCommand($batchCommand);


            if ($response->getHttpStatusCode() != 200 ||
                $response->getApiErrorCode() != 0 ||
                !empty($response->getHttpError())) {
                $result = "Код ответа API: {$response->getApiErrorCode()} Ответ: {$response->getApiErrorMessage()} {$response->getHttpError()}"
                          ." HTTP Код {$response->getHttpStatusCode()}";
                $error = 1;
            }

            $plugin->writeLog('response', $response);
            $plugin->writeLog('request', $response->getRequest());
        } catch (ChekonlineLineException $exception) {
            $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nLine:\n" . var_export($exception->getReceiptLine()->getParam(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            $plugin->writeLog('error', $logMessage);
            $error = 1;
            $result = $exception->getMessage();
        } catch (ChekonlineCommandException $exception) {
            $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nCommand:" . var_export($exception->getCommand()->getParams(), true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            $plugin->writeLog('error', $logMessage);
            $error = 1;
            $result = $exception->getMessage();
        } catch (ChekonlineSDKException $exception) {
            $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            $plugin->writeLog('error', $logMessage);
            $error = 1;
            $result = $exception->getMessage();
        }

        $this->response = array('message' => $result, 'error' => $error);
    }

}